@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -38,11 +38,13 @@
 
 #include "common.h"
 #include "storage.h"
+#include "idmap.h"
 
 #define DATA_CONNECTION_MANAGER_INTERFACE "org.ofono.DataConnectionManager"
 #define DATA_CONTEXT_INTERFACE "org.ofono.PrimaryDataContext"
 
 #define GPRS_FLAG_ATTACHING 0x1
+#define GPRS_FLAG_RECHECK 0x2
 
 #define SETTINGS_STORE "gprs"
 #define SETTINGS_GROUP "Settings"
@@ -70,9 +72,9 @@ struct ofono_gprs {
 	int cellid;
 	int technology;
 	int flags;
-	int next_context_id;
-	int cid_min;
-	int cid_max;
+	struct idmap *pid_map;
+	unsigned int last_context_id;
+	struct idmap *cid_map;
 	int netreg_status;
 	struct ofono_netreg *netreg;
 	unsigned int netreg_watch;
@@ -107,6 +109,7 @@ struct pri_context {
 	ofono_bool_t active;
 	enum gprs_context_type type;
 	char name[MAX_CONTEXT_NAME_LENGTH + 1];
+	unsigned int id;
 	char *path;
 	char *key;
 	struct context_settings *settings;
@@ -142,6 +145,16 @@ static enum gprs_context_type gprs_context_string_to_type(const char *str)
 	return GPRS_CONTEXT_TYPE_INVALID;
 }
 
+static unsigned int gprs_cid_alloc(struct ofono_gprs *gprs)
+{
+	return idmap_alloc(gprs->cid_map);
+}
+
+static void gprs_cid_release(struct ofono_gprs *gprs, unsigned int id)
+{
+	idmap_put(gprs->cid_map, id);
+}
+
 static struct pri_context *gprs_context_by_path(struct ofono_gprs *gprs,
 						const char *ctx_path)
 {
@@ -404,10 +417,14 @@ static void pri_activate_callback(const struct ofono_error *error,
 	DBG("%p %s", ctx, interface);
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Activating context failed with error: %s",
+		DBG("Activating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
+
+		gprs_cid_release(ctx->gprs, ctx->context.cid);
+		ctx->context.cid = 0;
+
 		return;
 	}
 
@@ -427,7 +444,6 @@ static void pri_activate_callback(const struct ofono_error *error,
 						DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
-
 }
 
 static void pri_deactivate_callback(const struct ofono_error *error, void *data)
@@ -438,13 +454,16 @@ static void pri_deactivate_callback(const struct ofono_error *error, void *data)
 	dbus_bool_t value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Deactivating context failed with error: %s",
+		DBG("Deactivating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
 		return;
 	}
 
+	gprs_cid_release(ctx->gprs, ctx->context.cid);
+	ctx->context.cid = 0;
+
 	ctx->active = FALSE;
 	__ofono_dbus_pending_reply(&gc->pending,
 				dbus_message_new_method_return(gc->pending));
@@ -634,6 +653,11 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 	if (g_str_equal(property, "Active")) {
 		struct ofono_gprs_context *gc = ctx->gprs->context_driver;
 
+		if (gc == NULL || gc->driver->activate_primary == NULL ||
+				gc->driver->deactivate_primary == NULL ||
+				ctx->gprs->cid_map == NULL)
+			return __ofono_error_not_implemented(msg);
+
 		if (gc->pending)
 			return __ofono_error_busy(msg);
 
@@ -651,15 +675,26 @@ static DBusMessage *pri_set_property(DBusConnection *conn,
 		if (ctx->gprs->flags & GPRS_FLAG_ATTACHING)
 			return __ofono_error_attach_in_progress(msg);
 
-		if (gc == NULL || gc->driver->activate_primary == NULL ||
-				gc->driver->deactivate_primary == NULL)
-			return __ofono_error_not_implemented(msg);
+		if (value) {
+			ctx->context.cid = gprs_cid_alloc(ctx->gprs);
+
+			if (ctx->context.cid == 0)
+				return __ofono_error_failed(msg);
+
+			if (ctx->context.cid !=
+					idmap_get_min(ctx->gprs->cid_map)) {
+				ofono_error("Multiple active contexts are"
+						" not yet supported");
+
+				gprs_cid_release(ctx->gprs, ctx->context.cid);
+				ctx->context.cid = 0;
+
+				return __ofono_error_failed(msg);
+			}
+		}
 
 		gc->pending = dbus_message_ref(msg);
 
-		/* TODO: Find lowest unused CID */
-		ctx->context.cid = 1;
-
 		if (value)
 			gc->driver->activate_primary(gc, &ctx->context,
 						pri_activate_callback, ctx);
@@ -760,17 +795,17 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	char path[256];
-	unsigned int id = ctx->gprs->next_context_id;
 	const char *basepath;
 
 	basepath = __ofono_atom_get_path(ctx->gprs->atom);
 
-	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, id);
+	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, ctx->id);
 
 	if (!g_dbus_register_interface(conn, path, DATA_CONTEXT_INTERFACE,
 					context_methods, context_signals,
 					NULL, ctx, pri_context_destroy)) {
 		ofono_error("Could not register PrimaryContext %s", path);
+		idmap_put(ctx->gprs->pid_map, ctx->id);
 		pri_context_destroy(ctx);
 
 		return FALSE;
@@ -779,8 +814,6 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 	ctx->path = g_strdup(path);
 	ctx->key = ctx->path + strlen(basepath) + 1;
 
-	ctx->gprs->next_context_id += 1;
-
 	return TRUE;
 }
 
@@ -790,6 +823,7 @@ static gboolean context_dbus_unregister(struct pri_context *ctx)
 	char path[256];
 
 	strcpy(path, ctx->path);
+	idmap_put(ctx->gprs->pid_map, ctx->id);
 
 	return g_dbus_unregister_interface(conn, path,
 						DATA_CONTEXT_INTERFACE);
@@ -902,6 +936,30 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 
 	gprs->attached = attached;
 
+	if (gprs->attached == FALSE) {
+		GSList *l;
+		struct pri_context *ctx;
+
+		for (l = gprs->contexts; l; l = l->next) {
+			ctx = l->data;
+
+			if (ctx->active == FALSE)
+				continue;
+
+			gprs_cid_release(gprs, ctx->context.cid);
+			ctx->context.cid = 0;
+
+			ctx->active = FALSE;
+			pri_reset_context_settings(ctx);
+
+			value = FALSE;
+			ofono_dbus_signal_property_changed(conn, ctx->path,
+						DATA_CONTEXT_INTERFACE,
+						"Active", DBUS_TYPE_BOOLEAN,
+						&value);
+		}
+	}
+
 	path = __ofono_atom_get_path(gprs->atom);
 	value = attached;
 	ofono_dbus_signal_property_changed(conn, path,
@@ -919,16 +977,21 @@ static void registration_status_cb(const struct ofono_error *error,
 		ofono_gprs_status_notify(gprs, status, lac, ci, tech);
 
 	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_attach_callback(const struct ofono_error *error, void *data)
 {
 	struct ofono_gprs *gprs = data;
 
-	if (error->type == OFONO_ERROR_TYPE_NO_ERROR) {
-		gprs->driver_attached = !gprs->driver_attached;
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
 		gprs_attached_update(gprs);
-	}
+	else
+		gprs->driver_attached = !gprs->driver_attached;
 
 	if (gprs->driver->registration_status) {
 		gprs->driver->registration_status(gprs, registration_status_cb,
@@ -937,6 +1000,11 @@ static void gprs_attach_callback(const struct ofono_error *error, void *data)
 	}
 
 	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_netreg_update(struct ofono_gprs *gprs)
@@ -953,12 +1021,15 @@ static void gprs_netreg_update(struct ofono_gprs *gprs)
 	if (gprs->driver_attached == attach)
 		return;
 
-	if (gprs->flags & GPRS_FLAG_ATTACHING)
+	if (gprs->flags & GPRS_FLAG_ATTACHING) {
+		gprs->flags |= GPRS_FLAG_RECHECK;
 		return;
+	}
 
 	gprs->flags |= GPRS_FLAG_ATTACHING;
 
 	gprs->driver->set_attached(gprs, attach, gprs_attach_callback, gprs);
+	gprs->driver_attached = attach;
 }
 
 static void netreg_status_changed(int status, int lac, int ci, int tech,
@@ -1113,8 +1184,9 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 		}
 
 		gprs_netreg_update(gprs);
-	} else
+	} else {
 		return __ofono_error_invalid_args(msg);
+	}
 
 	path = __ofono_atom_get_path(gprs->atom);
 	ofono_dbus_signal_property_changed(conn, path,
@@ -1134,6 +1206,7 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	const char *path;
 	enum gprs_context_type type;
 	char **objpath_list;
+	unsigned int id;
 
 	if (!dbus_message_get_args(msg, NULL, DBUS_TYPE_STRING, &name,
 					DBUS_TYPE_STRING, &typestr,
@@ -1148,23 +1221,31 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
 		return __ofono_error_invalid_format(msg);
 
-	if (g_slist_length(gprs->contexts) >= MAX_CONTEXTS)
+	if (gprs->last_context_id)
+		id = idmap_alloc_next(gprs->pid_map, gprs->last_context_id);
+	else
+		id = idmap_alloc(gprs->pid_map);
+
+	if (id > idmap_get_max(gprs->pid_map))
 		return __ofono_error_not_supported(msg);
 
 	context = pri_context_create(gprs, name, type);
+	context->id = id;
 
 	if (!context) {
 		ofono_error("Unable to allocate context struct");
 		return __ofono_error_failed(msg);
 	}
 
-	ofono_debug("Registering new context");
+	DBG("Registering new context");
 
 	if (!context_dbus_register(context)) {
 		ofono_error("Unable to register primary context");
 		return __ofono_error_failed(msg);
 	}
 
+	gprs->last_context_id = id;
+
 	if (gprs->settings) {
 		g_key_file_set_string(gprs->settings, context->key,
 					"Name", context->name);
@@ -1208,7 +1289,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 	char **objpath_list;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Removing context failed with error: %s",
+		DBG("Removing context failed with error: %s",
 				telephony_error_to_str(error));
 
 		__ofono_dbus_pending_reply(&gprs->pending,
@@ -1274,7 +1355,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 		storage_sync(gprs->imsi, SETTINGS_STORE, gprs->settings);
 	}
 
-	ofono_debug("Unregistering context: %s\n", ctx->path);
+	DBG("Unregistering context: %s\n", ctx->path);
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
 
@@ -1342,8 +1423,7 @@ void ofono_gprs_status_notify(struct ofono_gprs *gprs,
 				int status, int lac, int ci, int tech)
 {
 	/* If we are not attached and haven't tried to attach, ignore */
-	if (gprs->driver_attached == FALSE &&
-			(gprs->flags & GPRS_FLAG_ATTACHING) == 0)
+	if (gprs->driver_attached == FALSE)
 		return;
 
 	if (gprs->status != status)
@@ -1361,13 +1441,16 @@ void ofono_gprs_status_notify(struct ofono_gprs *gprs,
 	gprs_attached_update(gprs);
 }
 
-void ofono_gprs_set_cid_range(struct ofono_gprs *gprs, int min, int max)
+void ofono_gprs_set_cid_range(struct ofono_gprs *gprs,
+				unsigned int min, unsigned int max)
 {
 	if (gprs == NULL)
 		return;
 
-	gprs->cid_min = min;
-	gprs->cid_max = max;
+	if (gprs->cid_map)
+		idmap_free(gprs->cid_map);
+
+	gprs->cid_map = idmap_new_from_range(min, max);
 }
 
 static void gprs_context_unregister(struct ofono_atom *atom)
@@ -1389,7 +1472,8 @@ void ofono_gprs_add_context(struct ofono_gprs *gprs,
 	__ofono_atom_register(gc->atom, gprs_context_unregister);
 }
 
-void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc, unsigned cid)
+void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
+					unsigned int cid)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 	GSList *l;
@@ -1405,7 +1489,11 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc, unsigned cid)
 		if (ctx->context.cid != cid)
 			continue;
 
+		gprs_cid_release(ctx->gprs, ctx->context.cid);
+		ctx->context.cid = 0;
+
 		ctx->active = FALSE;
+		pri_reset_context_settings(ctx);
 
 		value = FALSE;
 		ofono_dbus_signal_property_changed(conn, ctx->path,
@@ -1547,6 +1635,11 @@ static void gprs_unregister(struct ofono_atom *atom)
 
 	g_slist_free(gprs->contexts);
 
+	if (gprs->cid_map) {
+		idmap_free(gprs->cid_map);
+		gprs->cid_map = NULL;
+	}
+
 	if (gprs->netreg_watch) {
 		if (gprs->status_watch) {
 			__ofono_netreg_remove_status_watch(gprs->netreg,
@@ -1573,6 +1666,11 @@ static void gprs_remove(struct ofono_atom *atom)
 	if (gprs == NULL)
 		return;
 
+	if (gprs->pid_map) {
+		idmap_free(gprs->pid_map);
+		gprs->pid_map = NULL;
+	}
+
 	if (gprs->context_driver) {
 		gprs->context_driver->gprs = NULL;
 		gprs->context_driver = NULL;
@@ -1620,7 +1718,7 @@ struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 	gprs->technology = -1;
 	gprs->cellid = -1;
 	gprs->location = -1;
-	gprs->next_context_id = 1;
+	gprs->pid_map = idmap_new(MAX_CONTEXTS);
 
 	return gprs;
 }
@@ -1655,64 +1753,75 @@ static gboolean load_context(struct ofono_gprs *gprs, const char *group)
 	gboolean ret = FALSE;
 	struct pri_context *context;
 	enum gprs_context_type type;
+	unsigned int id;
+
+	if (sscanf(group, "primarycontext%d", &id) != 1)
+		goto error;
+
+	if (id < 1 || id > MAX_CONTEXTS)
+		goto error;
 
 	if ((name = g_key_file_get_string(gprs->settings, group,
 					"Name", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	if ((typestr = g_key_file_get_string(gprs->settings, group,
 					"Type", NULL)) == NULL)
-		goto err;
+		goto error;
 
 	type = gprs_context_string_to_type(typestr);
 	if (type == GPRS_CONTEXT_TYPE_INVALID)
-		goto err;
+		goto error;
 
 	username = g_key_file_get_string(gprs->settings, group,
 						"Username", NULL);
 	if (!username)
-		goto err;
+		goto error;
 
 	if (strlen(username) > OFONO_GPRS_MAX_USERNAME_LENGTH)
-		goto err;
+		goto error;
 
 	password = g_key_file_get_string(gprs->settings, group,
 						"Password", NULL);
 
 	if (!password)
-		goto err;
+		goto error;
 
 	if (strlen(password) > OFONO_GPRS_MAX_PASSWORD_LENGTH)
-		goto err;
+		goto error;
 
 	apn = g_key_file_get_string(gprs->settings, group,
 					"AccessPointName", NULL);
 
 	if (!apn)
-		goto err;
+		goto error;
 
 	if (strlen(apn) > OFONO_GPRS_MAX_APN_LENGTH)
-		goto err;
+		goto error;
 
 	/* Accept empty (just created) APNs, but don't allow other
 	 * invalid ones */
 	if (apn[0] != '\0' && is_valid_apn(apn) == FALSE)
-		goto err;
+		goto error;
 
 	if ((context = pri_context_create(gprs, name, type)) == NULL)
-		goto err;
+		goto error;
 
+	idmap_take(gprs->pid_map, id);
+	context->id = id;
 	strcpy(context->context.username, username);
 	strcpy(context->context.password, password);
 	strcpy(context->context.apn, apn);
 
 	if (context_dbus_register(context) == FALSE)
-		goto err;
+		goto error;
+
+	gprs->last_context_id = id;
 
 	gprs->contexts = g_slist_append(gprs->contexts, context);
 	ret = TRUE;
 
-err:
+error:
 	g_free(name);
 	g_free(typestr);
 	g_free(username);
