@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -246,7 +246,7 @@ static void network_operator_populate_registered(struct ofono_netreg *netreg,
 				__ofono_atom_get_path(netreg->atom));
 
 	if (!dbus_connection_list_registered(conn, path, &children)) {
-		ofono_debug("Unable to obtain registered NetworkOperator(s)");
+		DBG("Unable to obtain registered NetworkOperator(s)");
 		*network_operators = g_try_new0(char *, 1);
 		return;
 	}
@@ -564,11 +564,11 @@ static void set_network_operator_eons_info(struct network_operator_data *opd,
 		oldinfo = "";
 
 	if (eons_info && eons_info->info)
-		newname = eons_info->info;
+		newinfo = eons_info->info;
 	else
 		newinfo = "";
 
-	if (oldname != newname && strcmp(oldname, newname))
+	if (oldinfo != newinfo && strcmp(oldinfo, newinfo))
 		ofono_dbus_signal_property_changed(conn, path,
 						NETWORK_OPERATOR_INTERFACE,
 						"AdditionalInformation",
@@ -948,8 +948,9 @@ void __ofono_netreg_set_base_station_name(struct ofono_netreg *netreg,
 		 * but don't emit signal */
 		if (netreg->current_operator == NULL)
 			return;
-	} else
+	} else {
 		netreg->base_station = g_strdup(name);
+	}
 
 	ofono_dbus_signal_property_changed(conn, path,
 						NETWORK_REGISTRATION_INTERFACE,
@@ -1152,7 +1153,7 @@ static void operator_list_callback(const struct ofono_error *error, int total,
 	gboolean need_to_emit;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error occurred during operator list");
+		DBG("Error occurred during operator list");
 		__ofono_dbus_pending_reply(&netreg->pending,
 					__ofono_error_failed(netreg->pending));
 		return;
@@ -1193,7 +1194,7 @@ static void current_operator_callback(const struct ofono_error *error,
 	DBG("%p, %p", netreg, netreg->current_operator);
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error during current operator");
+		DBG("Error during current operator");
 		return;
 	}
 
@@ -1272,7 +1273,7 @@ static void registration_status_callback(const struct ofono_error *error,
 	struct ofono_netreg *netreg = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error during registration status query");
+		DBG("Error during registration status query");
 		return;
 	}
 
@@ -1286,7 +1287,7 @@ static void init_registration_status(const struct ofono_error *error,
 	struct ofono_netreg *netreg = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error during registration status query");
+		DBG("Error during registration status query");
 		return;
 	}
 
@@ -1345,7 +1346,7 @@ static void signal_strength_callback(const struct ofono_error *error,
 	struct ofono_netreg *netreg = data;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Error during signal strength query");
+		DBG("Error during signal strength query");
 		return;
 	}
 
@@ -1606,8 +1607,14 @@ static void netreg_unregister(struct ofono_atom *atom)
 	__ofono_watchlist_free(netreg->status_watches);
 	netreg->status_watches = NULL;
 
-	for (l = netreg->operator_list; l; l = l->next)
+	for (l = netreg->operator_list; l; l = l->next) {
+		struct network_operator_data *opd = l->data;
+
+		if (opd->mcc[0] == '\0' && opd->mnc[0] == '\0')
+			continue;
+
 		network_operator_dbus_unregister(netreg, l->data);
+	}
 
 	g_slist_free(netreg->operator_list);
 	netreg->operator_list = NULL;
