@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -23,6 +23,8 @@
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 
+#include <ofono/types.h>
+
 void __ofono_exit();
 
 int __ofono_manager_init();
@@ -33,11 +35,9 @@ void __ofono_modem_shutdown();
 
 #include <ofono/log.h>
 
-int __ofono_log_init(gboolean detach, gboolean debug);
+int __ofono_log_init(const char *debug, ofono_bool_t detach);
 void __ofono_log_cleanup(void);
 
-void __ofono_toggle_debug(void);
-
 #include <ofono/dbus.h>
 
 int __ofono_dbus_init(DBusConnection *conn);
@@ -61,8 +61,6 @@ void __ofono_dbus_pending_reply(DBusMessage **msg, DBusMessage *reply);
 
 gboolean __ofono_dbus_valid_object_path(const char *path);
 
-#include <ofono/types.h>
-
 struct ofono_watchlist_item {
 	unsigned int id;
 	void *notify;
@@ -115,6 +113,7 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPES_CALL_VOLUME = 15,
 	OFONO_ATOM_TYPE_GPRS = 16,
 	OFONO_ATOM_TYPE_GPRS_CONTEXT = 17,
+	OFONO_ATOM_TYPE_RADIO_SETTINGS = 18,
 };
 
 enum ofono_atom_watch_condition {
@@ -170,6 +169,7 @@ void __ofono_atom_free(struct ofono_atom *atom);
 #include <ofono/voicecall.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
+#include <ofono/radio-settings.h>
 
 #include <ofono/sim.h>
 
